# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def multivalue_subset_config(self, source_protocol_handle, destination_protocol_handle, target_attribute, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    multivalue_subset_config
		
		 Description:
		    This is an internal method that is used to set multivalue attributes to a subset pattern directly.
		
		 Synopsis:
		    multivalue_subset_config
		x       -source_protocol_handle       ANY
		x       [-source_node_handle          ANY]
		x       -destination_protocol_handle  ANY
		x       [-destination_node_handle     ANY]
		x       -target_attribute             ANY
		
		 Arguments:
		x   -source_protocol_handle
		x       An NGPF handle that was returned by a previous HLT command that identifies
		x       the protocol stack that will be used as a subset source.
		x   -source_node_handle
		x       An XPath expression that identifies the autogenerated node that will be used
		x       as a subset source.
		x   -destination_protocol_handle
		x       An NGPF handle that was returned by a previous HLT command that identifies
		x       the protocol stack that will be used as a subset destination.
		x   -destination_node_handle
		x       An XPath expression that identifies the autogenerated node that will be used
		x       as a subset destination.
		x   -target_attribute
		x       The name of the attribute that will be modified.
		
		 Return Values:
		    $::SUCCESS | $::FAILURE
		    key:status  value:$::SUCCESS | $::FAILURE
		    On status of failure, gives detailed information.
		    key:log     value:On status of failure, gives detailed information.
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'multivalue_subset_config', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
